#include "testApp.h"

bool testApp::isPointInsidePolygon(ofPoint *polygon, int N, ofPoint p) {
	int counter = 0;
	int i;
	double xinters;
	ofPoint p1,p2;
	
	p1 = polygon[0];
	
	for (i=1;i<=N;i++) {
		p2 = polygon[i % N];
		if (p.y > MIN(p1.y,p2.y)) {
			if (p.y <= MAX(p1.y,p2.y)) {
				if (p.x <= MAX(p1.x,p2.x)) {
					if (p1.y != p2.y) {
						xinters = (p.y-p1.y)*(p2.x-p1.x)/(p2.y-p1.y)+p1.x;
						if (p1.x == p2.x || p.x <= xinters)
							counter++;
					}
				}
			}
		}
		p1 = p2;
	}
	
	if (counter % 2 == 0)
		return false;
	else
		return true;
} 


//--------------------------------------------------------------
void testApp::setup() {
	ofSetFrameRate(60);
	ofSetVerticalSync(true);
	ofBackground( 30, 30, 30);
	
	numFonts = directory.listDir("fonts/");
	fontIndex	= floor(3);
	font.loadFont(directory.getPath(fontIndex), 300, true, true, true, .5);
	// loadFont(string filename, int fontsize, bool _bAntiAliased=true, bool _bFullCharacterSet=false, bool makeContours=false, float simplifyAmt=0.3, int dpi=0);
	
	fbo.allocate((float)ofGetWidth()*.3, (float)ofGetHeight()*.3);
	
	oldLetter	= ' ';
	letter		= 'n';
	
	samplePixels		= 33.;
	bChangeSamplePixels = true;
	
	cout << "tris size: " << delauney.triangles.size() << endl;
}

//--------------------------------------------------------------
void testApp::update() {
	
	if(letter != oldLetter || bChangeSamplePixels) {
		tChar = font.getCharacterAsPoints(letter);
		points.clear();
		delauney.reset();
		tris.clear();
		sampledPolys.clear();
		
		centroids.clear();
		origTris.clear();
		bTriRemoved.clear();
		
		
		float fw = font.stringWidth( ofToString(letter) );
		float fh = font.stringHeight(ofToString(letter));
		fbo.allocate(fw, fh);
		fbo.begin();
		ofSetColor(0, 0, 0);
		ofRect(0, 0, fbo.getWidth(), fbo.getHeight());
		
		ofSetColor(255, 255, 255);
		ofRectangle bounds = font.getStringBoundingBox( ofToString(letter), 0, 0);
		font.drawStringAsShapes(ofToString(letter), -bounds.x, -bounds.y);
		fbo.end();
		
		ofPixels pix;
		fbo.readToPixels(pix);
		
		ofPath path	= font.getCharacterAsPoints(letter);
		vector <ofPolyline> polys = path.getOutline();
		// resample the font points so that we have a point every (x) pixels;
		for (int i = 0; i < polys.size(); i++) {
			
			// now resample and draw new dots
			ofPolyline sampled = polys[i].getResampledBySpacing(samplePixels);
			sampledPolys.push_back( sampled );
			// outline
			for (int j = 0; j < sampled.size(); j++) {
				points.push_back( ofVec3f(sampled[j].x, sampled[j].y, 0.));
			}
		}
		
		delauney.reset();
		for (int i = 0; i < points.size(); i++) {
			delauney.addPoint(points[i].x, points[i].y, points[i].z);
		}
		delauney.triangulate();
		
		ofPoint* srcPts = new ofPoint[points.size()];
		for (int i = 0; i < points.size(); i++) {
			srcPts[i].x = points[i].x;
			srcPts[i].y = points[i].y;
			srcPts[i].z = points[i].z;
		}
		
		ofPoint centroid;
		for( int i=0; i < delauney.triangles.size(); i++ ) {
			ofxDelaunayTriangle& triangle = delauney.triangles[ i ];
			centroid.set(0, 0, 0);
			for(int j = 0; j < 3; j++) {
				centroid += triangle.points[j];
			}
			centroid /= 3.f;
			
			origTris.push_back( ofxDelaunayTriangle() );
			origTris[origTris.size()-1].points[0] = triangle.points[0];
			origTris[origTris.size()-1].points[1] = triangle.points[1];
			origTris[origTris.size()-1].points[2] = triangle.points[2];
			
			centroids.push_back( centroid );
			
			ofColor col = pix.getColor(centroid.x-bounds.x, centroid.y-bounds.y);
			
			//cout << "centroid " << centroid.x
			
			if(col.r > 200) {
			//if(isPointInsidePolygon( srcPts, points.size(), centroid)) {
				tris.push_back( ofxDelaunayTriangle() );
				for(int t = 0; t < 3; t++) {
					tris[tris.size()-1].points[t] = triangle.points[t];
					tris[tris.size()-1].pointIndex[t] = triangle.pointIndex[t];
				}
				
				bTriRemoved.push_back( false );
			} else {
				bTriRemoved.push_back( true );
			}
		}
		delete srcPts;
		
		buildMesh();
		
		oldLetter = letter;
		bChangeSamplePixels = false;
	}
}

//--------------------------------------------------------------
void testApp::draw() {
	
	ofSetLineWidth(1.f);
	
	ofSetColor(50, 50, 50);
	
	float pct;
	float xoffset = 20.;
	float yoffset = 70.;
	float fw = font.stringWidth( ofToString(letter) );
	float fh = font.stringHeight(ofToString(letter));
	ofRectangle bounds = font.getStringBoundingBox( ofToString(letter), 0, 0);
	
	ofNoFill();
	glPushMatrix();
	glTranslatef(xoffset-bounds.x, yoffset+(-bounds.y), 0);
	for(int i = 0; i < origTris.size(); i++) {
		ofxDelaunayTriangle& triangle = origTris[ i ];
		
		ofSetColor(255, 100, 200);
		
		glBegin(GL_LINES);
		for(int j = 0; j < 2; j++) {
			glVertex3f( triangle.points[j].x, triangle.points[j].y, triangle.points[j].z);
			glVertex3f( triangle.points[j+1].x, triangle.points[j+1].y, triangle.points[j+1].z);
		}
		glVertex3f( triangle.points[2].x, triangle.points[2].y, triangle.points[2].z);
		glVertex3f( triangle.points[0].x, triangle.points[0].y, triangle.points[0].z);
		
		glEnd();
		ofFill();
		ofSetColor(255, 255, 0);
		if(bTriRemoved[i]) ofSetColor(255, 50, 10);
		ofCircle(centroids[i].x, centroids[i].y, 2.f);
		ofNoFill();
	}
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(xoffset+fw+10-bounds.x, yoffset-bounds.y, 0);
	ofSetColor(255, 0, 255);
	for( int i=0; i < tris.size(); i++ ) {
		ofxDelaunayTriangle& triangle = tris[ i ];
		
		glBegin(GL_LINES);
		for(int j = 0; j < 2; j++) {
			glVertex3f( triangle.points[j].x, triangle.points[j].y, triangle.points[j].z);
			glVertex3f( triangle.points[j+1].x, triangle.points[j+1].y, triangle.points[j+1].z);
		}
		glVertex3f( triangle.points[2].x, triangle.points[2].y, triangle.points[2].z);
		glVertex3f( triangle.points[0].x, triangle.points[0].y, triangle.points[0].z);
		
		glEnd();
	}
	glPopMatrix();
	
	
	ofPath path	= font.getCharacterAsPoints(letter);
	vector <ofPolyline> polys = path.getOutline();
	glPushMatrix();
	glTranslatef(xoffset+(fw*2)+20-bounds.x, yoffset-bounds.y, 0.);
	// resample the font points so that we have a point every (x) pixels;
	for(int i = 0; i < sampledPolys.size(); i++) {
		// now resample and draw new dots
		ofPolyline sampled = sampledPolys[i];
		ofSetColor(255, 0, 0);
		if(i > 0) ofSetColor(255, 255, 9);
		//ofPushMatrix();
		//ofTranslate(475, yoffset, 0);
		sampled.draw();
		//ofPopMatrix();
	}
	glPopMatrix();
	ofFill();
	
	
	
	
	ofSetColor(255, 255, 255);
	glPushMatrix();
	glTranslatef(xoffset, yoffset+fh+60, 0);
	fbo.draw(0, 0);
	glPopMatrix();
	
	glEnable(GL_DEPTH_TEST);
	glPushMatrix();
	glTranslatef(xoffset+fbo.getWidth()+10, yoffset+fh+fh+60, 0);
	ofSetColor(255, 255, 255);
	mesh.drawVertices();
	ofSetColor(40, 100, 130);
	mesh.drawWireframe();
	glPopMatrix();
	
	glPushMatrix();
	glTranslatef(xoffset+fbo.getWidth()+30+fw, yoffset+fh+fh+60, 0);
	ofSetColor(200, 200, 200);
	mesh.draw();
	glPopMatrix();
	
	glDisable(GL_DEPTH_TEST);
	
	
	stringstream ss;
	ss << "sample pixels(up/down): " << samplePixels << endl;
	ss << "fonts(left/right): " << directory.getName(fontIndex) << "  " << (fontIndex+1) << " / " << numFonts << endl;
	ofSetColor(255, 255, 255);
	ofDrawBitmapString(ss.str().c_str(), 10, 10);
}

//--------------------------------------------------------------
void testApp::buildMesh() {
	mesh.clearVertices();
	mesh.clearIndices();
	mesh.clear();
	float newz = 50.f;
	ofVec3f p1, p2, p3;
	int offsetindex = 0;
	
	// loop around base of text //
	for(int i = 0; i < sampledPolys.size(); i++) {
		for (int j = 0; j < sampledPolys[i].size(); j++) {
			p1.set(sampledPolys[i][j].x, sampledPolys[i][j].y, 0.);
			mesh.addVertex(p1);
		}
	}
	offsetindex = mesh.getNumVertices();
	// then add vertexes for the extruded text //
	for(int i = 0; i < sampledPolys.size(); i++) {
		for (int j = 0; j < sampledPolys[i].size(); j++) {
			p1.set(sampledPolys[i][j].x, sampledPolys[i][j].y, newz);
			mesh.addVertex(p1);
		}
	}
	
	ofIndexType i1, i2, i3;
	int ip	= 0;
	int ip2 = offsetindex;
	for(int i = 0; i < sampledPolys.size(); i++) {
		for (int j = 0; j < sampledPolys[i].size()-1; j++) {
			i1 = j + ip2;
			i2 = j+1 + ip2;
			i3 = j + ip;
			mesh.addTriangle(i1, i2, i3);
			i1 = j + ip;
			i2 = j+1+ip;
			i3 = j+1+ip2;
			mesh.addTriangle(i1, i2, i3);
		}
		
		i1 = ip;
		i2 = ip+sampledPolys[i].size()-1;
		i3 = ip2;
		mesh.addTriangle(i1, i2, i3);
		
		i1 = ip2;
		i2 = ip2 +sampledPolys[i].size()-1;
		i3 = ip + sampledPolys[i].size()-1;
		mesh.addTriangle(i1, i2, i3);
		
		
		ip += sampledPolys[i].size();
		ip2 += sampledPolys[i].size();
	}
	
	
	ip	= 0;
	ip2 = offsetindex;
	ofPoint tpts[3];
	bool bF[3];
	for( int i=0; i < tris.size(); i++ ) {
		ofxDelaunayTriangle& triangle = tris[ i ];// pointIndex
		tpts[0] = tris[i].points[0];
		tpts[1] = tris[i].points[1];
		tpts[2] = tris[i].points[2];
		bF[0] = bF[1] = bF[2] = false;
		for(int j = 0; j < points.size(); j++) {
			//for (int k = 0; k < sampledPolys[j].size(); k++) {
			for(int r = 0; r < 3; r++) {
				//if(sampledPolys[j][k].x == tpts[r].x && sampledPolys[j][k].y == tpts[r].y) {
				if(points[j].x == tpts[r].x && points[j].y == tpts[r].y) {
					tris[i].pointIndex[r] = j;
					bF[r]	= true;
				}
				if(bF[0] == true && bF[1] == true && bF[2] == true) break;
			}
		}
	}
	for( int i=0; i < tris.size(); i++ ) {
		mesh.addTriangle(tris[i].pointIndex[0], tris[i].pointIndex[1], tris[i].pointIndex[2]);
		mesh.addTriangle(tris[i].pointIndex[0]+offsetindex, tris[i].pointIndex[1]+offsetindex, tris[i].pointIndex[2]+offsetindex);
	}
	
	cout << "mesh verts= " << mesh.getNumVertices() << " indicies = " << mesh.getNumIndices() << endl;
}
 

//--------------------------------------------------------------
void testApp::keyPressed(int key) {
	cout << key << endl;
	if(key == OF_KEY_UP || key == 357) {
		samplePixels		+= .5;
		bChangeSamplePixels = true;
	} else if (key == OF_KEY_DOWN || key == 359) {
		samplePixels		-= .5;
		bChangeSamplePixels = true;
	} else if(key == OF_KEY_RIGHT) {
		fontIndex	++;
		if(fontIndex > numFonts-1) fontIndex = numFonts-1;
		font.loadFont(directory.getPath(fontIndex), 300, true, true, true, .5);
		bChangeSamplePixels = true;
	} else if (key == OF_KEY_LEFT) {
		fontIndex--;
		if(fontIndex < 0) fontIndex = 0;
		font.loadFont(directory.getPath(fontIndex), 300, true, true, true, .5);
		bChangeSamplePixels = true;
	} else if(key != ' ') {
		letter = key;
	}
	if(samplePixels < .01) samplePixels = 0.01;
}

//--------------------------------------------------------------
void testApp::keyReleased(int key) {
	
}

//--------------------------------------------------------------
void testApp::mouseMoved(int x, int y) {
	
}

//--------------------------------------------------------------
void testApp::mouseDragged(int x, int y, int button) {
	
}

//--------------------------------------------------------------
void testApp::mousePressed(int x, int y, int button) {
	
}

//--------------------------------------------------------------
void testApp::mouseReleased(int x, int y, int button){
	
}

//--------------------------------------------------------------
void testApp::windowResized(int w, int h) {
	
}

//--------------------------------------------------------------
void testApp::gotMessage(ofMessage msg) {
	
}

//--------------------------------------------------------------
void testApp::dragEvent(ofDragInfo dragInfo) { 
	
}